@@ -4,16 +4,40 @@
 #   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
 #   All rights reserved. Copyright (c) 2016-2020 VECTIONEER.
 #
+import base64
+import hashlib
+import json
+import tempfile
 
 from motorcortex.reply import Reply
+from motorcortex.setup_logger import logger
 
 import os
 import queue
-import ctypes
 from concurrent.futures import ThreadPoolExecutor
 from pynng import Req0, TLSConfig
 
 
+class Base64Encoder(json.JSONEncoder):
+    # pylint: disable=method-hidden
+    encoding = 'utf-8'
+
+    def default(self, o):
+        if isinstance(o, bytes):
+            return base64.b64encode(o).decode(Base64Encoder.encoding)
+        return json.JSONEncoder.default(self, o)
+
+
+class Base64Decoder(json.JSONDecoder):
+    # pylint: disable=method-hidden
+    encoding = 'utf-8'
+
+    def default(self, o):
+        if isinstance(o, bytes):
+            return base64.b64decode(o).encode(Base64Decoder.decode())
+        return json.Base64Decoder.default(self, o)
+
+
 class Request(object):
 
     def __init__(self, protobuf_types, parameter_tree):
@@ -329,7 +353,7 @@ class Request(object):
             param_value = protobuf_types.createType(type_name)
 
         if not param_value:
-            print("Failed to find encoder for the path: %s type: %s" % (path, type_name))
+            logger.error("Failed to find encoder for the path: %s type: %s" % (path, type_name))
 
         # creating type instance
         set_param_msg = protobuf_types.createType("motorcortex.SetParameterMsg")
@@ -358,7 +382,7 @@ class Request(object):
             param_value = protobuf_types.createType(type_name)
 
         if not param_value:
-            print("Failed to find encoder for the path: %s type: %s" % (path, type_name))
+            logger.error("Failed to find encoder for the path: %s type: %s" % (path, type_name))
 
         # creating type instance
         overwrite_param_msg = protobuf_types.createType("motorcortex.OverwriteParameterMsg")
@@ -404,67 +428,50 @@ class Request(object):
         except queue.Empty:
             return False
 
-    @staticmethod
-    def __hash(tree):
-        tree_hash = int()
-
-        def clip_u32(val):
-            return ctypes.c_uint32(val).value
-
-        for param in tree.params:
-            tree_hash += clip_u32(param.data_size + param.data_type + param.id + param.number_of_elements + \
-                                  param.param_type + param.unit + param.group_id + param.permissions)
-            for ch in param.path:
-                tree_hash = clip_u32((tree_hash << 5) - tree_hash) + ord(ch)
-                tree_hash |= 0
-
-            if hasattr(param, 'module_type'):
-                for ch in param.module_type:
-                    tree_hash = clip_u32((tree_hash << 3) - tree_hash) + ord(ch)
-
-        print(tree_hash)
-
-        return tree_hash
-
     @staticmethod
     def __getParameterTree(hash_reply, protobuf_types, socket):
         tree_hash = hash_reply.get()
-        tree = Request.__loadParameterTreeFile(str(tree_hash.hash), protobuf_types)
+        path = tempfile.gettempdir() + '/' + str(tree_hash)
+        tree = Request.loadParameterTreeFile(path, protobuf_types)
         if tree:
-            if tree_hash.hash == Request.__hash(tree):
-                print('Found parameter tree in the cache')
-                return tree
-            else:
-                print('Found parameter tree but the cache does not match')
+            logger.debug('Found parameter tree in the cache')
+            return tree
         else:
-            print('Failed to find parameter tree in the cache')
+            logger.debug('Failed to find parameter tree in the cache')
 
         # getting and instantiating data type from the loaded dict
         param_tree_msg = protobuf_types.createType('motorcortex.GetParameterTreeMsg')
         handle = Request.__send(socket, protobuf_types.encode(param_tree_msg), protobuf_types)
 
         # encoding and sending data
-        return Request.__saveParameterTreeFile(handle, tree_hash.hash)
+        return Request.saveParameterTreeFile(path, handle)
 
     @staticmethod
-    def __saveParameterTreeFile(parameter_tree, hash):
-        print('Saved parameter tree to the cache')
-        # cache = Request.__hash(parameter_tree)
-        # path = str(cache)
-        path = str(hash)
-        data = parameter_tree.SerializeToString()
-        text_file = open(path, "wb")
-        text_file.write(data)
-        text_file.close()
+    def saveParameterTreeFile(path, parameter_tree):
+        logger.debug('Saved parameter tree to the cache')
+        json_data = {}
+        base64_data = base64.b64encode(parameter_tree.SerializeToString())
+        json_data['md5'] = hashlib.md5(base64_data).hexdigest()
+        json_data['data'] = base64_data.decode('utf-8')
+
+        with open(path, "w") as outfile:
+            outfile.write(json.dumps(json_data))
+
         return parameter_tree
 
     @staticmethod
-    def __loadParameterTreeFile(path, protobuf_types):
+    def loadParameterTreeFile(path, protobuf_types):
+        logger.debug('Loaded parameter tree from the cache')
+        param_tree_hash_msg = None
         if os.path.exists(path):
-            text_file = open(path, "rb")
-            data = text_file.read()
-            text_file.close()
-            param_tree_hash_msg = protobuf_types.createType('motorcortex.ParameterTreeMsg')
-            param_tree_hash_msg.ParseFromString(data)
-            return param_tree_hash_msg
-        return None
+            with open(path, "r") as outfile:
+                json_data = json.load(outfile)
+
+            if json_data:
+                if "md5" in json_data and "data" in json_data:
+                    if hashlib.md5(json_data['data'].encode()).hexdigest() == json_data['md5']:
+                        param_tree_hash_msg = protobuf_types.createType('motorcortex.ParameterTreeMsg')
+                        tree_raw = base64.b64decode(json_data['data'])
+                        param_tree_hash_msg.ParseFromString(tree_raw)
+
+        return param_tree_hash_msg
